from datetime import datetime
from enum import auto
from typing import Any, Dict, Optional
from uuid import UUID

from encord.orm.analytics import CamelStrEnum
from encord.orm.base_dto import BaseDTO


class StorageUserRole(CamelStrEnum):
    USER = auto()
    ADMIN = auto()


class StorageFolder(BaseDTO):
    uuid: UUID
    parent: Optional[UUID]
    name: str
    description: str
    client_metadata: Optional[str]
    owner: str
    created_at: datetime
    last_edited_at: datetime
    user_role: StorageUserRole
    synced_dataset_hash: Optional[UUID]


class CreateStorageFolderPayload(BaseDTO):
    name: str
    description: Optional[str]
    parent: Optional[UUID]
    client_metadata: Optional[str]



from typing import Dict, Optional

import encord.orm.client_metadata_schema as orm
from encord.http.v2.api_client import ApiClient


def set_client_metadata_schema_from_dict(
    api_client: ApiClient, organisation_id: int, json_dict: Dict[str, orm.ClientMetadataSchemaTypes]
):
    try:
        validated_dict = {key: orm.ClientMetadataSchemaTypes(val) for key, val in json_dict.items()}
    except ValueError:
        raise NotImplementedError(
            f"Got an unexpected data type in schema. Valid data types are: "
            f"{', '.join([v for v in orm.ClientMetadataSchemaTypes])}."
        )
    payload = orm.ClientMetadataSchemaPayload(metadata_schema=validated_dict)
    api_client.post(
        f"organisation/{organisation_id}/client-metadata-schema", params=None, payload=payload, result_type=None
    )


def get_client_metadata_schema(
    api_client: ApiClient, organisation_id: int
) -> Optional[Dict[str, orm.ClientMetadataSchemaTypes]]:
    client_metadata_schema: Optional[orm.ClientMetadataSchema] = api_client.get(
        f"organisation/{organisation_id}/client-metadata-schema",
        params=None,
        result_type=orm.ClientMetadataSchema,
        allow_none=True,
    )
    return client_metadata_schema.metadata_schema if client_metadata_schema else None



import json
from typing import Any, Dict, Optional
from uuid import UUID

import encord.orm.storage as orm_storage
from encord.http.v2.api_client import ApiClient


class StorageFolder:
    def __init__(self, api_client: ApiClient, orm_folder: orm_storage.StorageFolder):
        self._api_client = api_client
        self._orm_folder = orm_folder
        self._parsed_metadata: Optional[Dict[str, Any]] = None

    @property
    def uuid(self) -> UUID:
        return self._orm_folder.uuid

    @property
    def parent_uuid(self) -> Optional[UUID]:
        return self._orm_folder.parent

    @property
    def parent(self) -> Optional["StorageFolder"]:
        parent_uuid = self._orm_folder.parent
        return None if parent_uuid is None else self._get_folder(self._api_client, parent_uuid)

    @property
    def name(self) -> str:
        return self._orm_folder.name

    @property
    def description(self) -> str:
        return self._orm_folder.description

    @property
    def client_metadata(self) -> Optional[Dict[str, Any]]:
        if self._parsed_metadata is None:
            if self._orm_folder.client_metadata is not None:
                self._parsed_metadata = json.loads(self._orm_folder.client_metadata)
        return self._parsed_metadata

    def delete(self):
        self._api_client.delete(f"storage/folders/{self.uuid}", params=None, result_type=None)

    @staticmethod
    def _get_folder(api_client: ApiClient, folder_uuid: UUID) -> "StorageFolder":
        orm_folder = api_client.get(
            f"storage/folders/{folder_uuid}", params=None, result_type=orm_storage.StorageFolder
        )
        return StorageFolder(api_client, orm_folder)
